import unittest
from autohandshake.src.Pages import AppointmentTypePage
from autohandshake.tests import TestSession

TEST_TYPE_ID = 16097


class TestAppointmentTypePage(unittest.TestCase):

    def test_get_appt_settings(self):
        expected_settings = {
            'id': TEST_TYPE_ID,
            'name': 'Homewood: TEST appointment type',
            'description': 'a malleable appointment type for testing purposes',
            'length': 30,
            'categories': ['Create Documents', 'Find Jobs/Internships/Opportunities'],
            'drop_in_enabled': True,
            'pre_message': 'This is a pre-appointment message',
            'pre_survey': 'Homewood: Pre- Appointment Survey',
            'post_message': 'This is the post-appointment message',
            'post_survey': 'Homewood: 2017 Mock Interview post-appointment survey',
            'staff_survey': 'Homewood: Post-Appointment Staff Survey',
            'school_years': ['Sophomore', 'Senior', 'Doctorate'],
            'cum_gpa_required': True,
            'cum_gpa': 3.5,
            'major_groups': ['major_group: Law', 'major_group: Public Policy',
                             'major_group: Individual Studies', 'indv_major: Applied Economics',
                             'indv_major: Applied Biomedical Engineering',
                             'indv_major: Applied and Computational Mathematics'],
            'colleges': ['Whiting School of Engineering', 'Krieger School of Arts & Sciences'],
            'labels': ['shared: appointments allowed', 'hwd: old appointment type'],
            'career_clusters': ['Homewood: STEM & Innovation', 'Carey Network: Finance']
        }

        with TestSession() as browser:
            type_page = AppointmentTypePage(TEST_TYPE_ID, browser)
            self.assertEqual(expected_settings, type_page.get_settings())



import unittest
from autohandshake import CareerInterestsPage, ViewAsStudent
from autohandshake.tests import TestSession
from autohandshake.src.constants import BASE_URL
from autohandshake.src.exceptions import InsufficientPermissionsError

TEST_STUDENT_ID = 20474668


class TestCareerInterestsPage(unittest.TestCase):

    def test_page_loads_correctly(self):
        with TestSession() as browser:
            try:
                page = CareerInterestsPage(TEST_STUDENT_ID, browser)
                self.fail("Did not throw permissions error when logged in as staff")
            except InsufficientPermissionsError:
                pass  # it should throw an error if not logged in as a student
            with ViewAsStudent(TEST_STUDENT_ID, browser):
                page = CareerInterestsPage(TEST_STUDENT_ID, browser)

    def test_selects_clusters_and_saves_correctly(self):
        with TestSession() as browser:
            with ViewAsStudent(TEST_STUDENT_ID, browser):
                page = CareerInterestsPage(TEST_STUDENT_ID, browser)
                finance_selected = False
                health_selected = False
                if page.cluster_is_selected_by_id(200):
                    finance_selected = True
                    page.deselect_cluster_by_id(200)
                if page.cluster_is_selected_by_id(201):
                    health_selected = True
                    page.deselect_cluster_by_name("Homewood: Health Sciences")

                self.assertFalse(page.cluster_is_selected_by_id(200))
                self.assertFalse(page.cluster_is_selected_by_id(201))

                page.select_cluster_by_id(200)  # Homewood: Finance
                page.select_cluster_by_name('Homewood: Health Sciences')  # id: 201

                self.assertTrue(page.cluster_is_selected_by_id(201))
                self.assertTrue(page.cluster_is_selected_by_name('Homewood: Finance'))

                # make sure "select" only clicks when the cluster isn't already selected
                page.select_cluster_by_id(200)
                self.assertTrue(page.cluster_is_selected_by_name('Homewood: Finance'))

                page.save_interests()

                # navigate away and back again to make sure it saved
                browser.get(BASE_URL)

                page = CareerInterestsPage(TEST_STUDENT_ID, browser)
                self.assertTrue(page.cluster_is_selected_by_id(200))
                self.assertTrue(page.cluster_is_selected_by_name('Homewood: Health Sciences'))

                # clean up the career interests so they are the way we found them
                if not finance_selected:
                    page.deselect_cluster_by_id(200)
                if not health_selected:
                    page.deselect_cluster_by_id(201)

                page.save_interests()



import unittest

from autohandshake import EventPage
from autohandshake.tests import TestSession


class TestEventPage(unittest.TestCase):

    def test_get_invited_schools(self):
        multischool_event_id = 592834
        with TestSession() as browser:
            page = EventPage(multischool_event_id, browser)
            schools = page.get_invited_schools()
            self.assertEqual(len(schools), 210)
            self.assertEqual(schools[0], 'Central Michigan University')
            self.assertEqual(schools[209], 'University of Illinois at Chicago')



from autohandshake.src.Pages import Page
from autohandshake.src.HandshakeBrowser import HandshakeBrowser, UserType
from autohandshake.src.constants import BASE_URL
from autohandshake.src.exceptions import NoSuchElementError
from typing import List
from bs4 import BeautifulSoup


class LabelModal:
    _MODAL_XPATH = '//div[@data-bind="with: selectedLabel"]'

    def __init__(self, browser: HandshakeBrowser, row_idx: int):
        self._browser = browser
        self._row_idx = row_idx

    def __enter__(self):
        ROW_XPATH = f'//tbody[@data-bind="foreach: institution_labels"]/tr[{self._row_idx + 1}]'
        self._browser.click_element_by_xpath(ROW_XPATH)
        self._browser.wait_until_element_is_clickable_by_xpath(self._MODAL_XPATH)
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        CLOSE_BTN_XPATH = f'{self._MODAL_XPATH}//button[@class="btn btn-default"]'
        self._browser.click_element_by_xpath(CLOSE_BTN_XPATH)
        self._browser.wait_until_element_does_not_exist_by_xpath(self._MODAL_XPATH)

    def get_usage_counts(self):
        usage_counts = {}
        try:
            table_rows = BeautifulSoup(self._browser.get_element_attribute_by_xpath(
                '//tbody[@data-bind="foreach: type_details"]', 'innerHTML'),
                'html.parser').find_all('tr', recursive=False)
            for row in table_rows:
                usage_counts = self._parse_row(row, usage_counts)
        except NoSuchElementError:
            usage_counts = {'All': 0}
        return usage_counts

    def _parse_row(self, row: BeautifulSoup, usage_counts: dict) -> dict:
        usage_type = row.find('a', {'data-bind': 'text: labellable_type'}).text
        usage_count = int(row.find('a', {'data-bind': 'text: count'}).text)
        usage_counts[usage_type] = usage_count
        return usage_counts


class LabelSettingsPage(Page):

    def __init__(self, browser: HandshakeBrowser):
        """
        :param browser: a logged-in HandshakeBrowser
        :type browser: HandshakeBrowser
        """
        labels_base_url = f'{BASE_URL}/schools/{browser.school_id}/institution_labels'
        url_query = '?ajax=true&query=&category=ManageLabels&page=1&per_page=1000&sort_direction=asc&sort_column=default&followed_only=false&qualified_only=&core_schools_only=false&including_all_facets_in_searches=false'
        super().__init__(labels_base_url + url_query,
                         browser)

    @Page.require_user_type(UserType.STAFF)
    def get_label_data(self) -> List[dict]:
        """
        Get a list of all the school's labels and their usage data.

        :return: a list of label data dicts, of the form:
        ::

            {
                'label_name': 'preferred employer', # the name of the label
                'usage_counts': { # how many times the label has been used in different system modules
                    'Job': 43,
                    'Event': 55,
                    'InterviewSchedule': 23
                },
                'used_for': 'All', # the primary module this label is used for. Labels used for multiple modules are marked 'All'
                'created_by_first_name': 'John', # the first name of the user who created the label
                'created_by_last_name': 'Staffmember', # the last name of the user who created the label
                'label_type': 'normal' # the label type. One of 'normal', 'system' or 'public'
            }
        :rtype: list
        """
        table_rows = BeautifulSoup(self._browser.get_element_attribute_by_xpath(
            '//tbody', 'innerHTML'),
            'html.parser').find_all('tr', recursive=False)
        return self._parse_table(table_rows)

    def _parse_table(self, table_rows: List[BeautifulSoup]) -> List[dict]:
        label_data = []
        for idx, row in enumerate(table_rows):
            label_data.append(self._parse_table_row(row, idx))
        return label_data

    def _parse_table_row(self, row: BeautifulSoup, row_idx: int) -> dict:
        used_for = row.find('a', {'data-bind': 'text: labellable_type_text, click: $parent.searchByLabels'}).text
        return {
            'label_name': row.find('a', {'data-bind': 'text: name, click: $parent.searchByLabels'}).text,
            'usage_counts': self._get_usage_counts(row, row_idx, used_for),
            'used_for': used_for,
            'created_by_first_name': self._get_first_name(row),
            'created_by_last_name': self._get_last_name(row),
            'label_type': row.find('div', {
                'data-bind': 'text: label_type, css: label_type_css, click: $parent.searchByLabels'}).text
        }

    def _get_usage_counts(self, row: BeautifulSoup, row_idx: int, used_for: str) -> dict:
        usage_count = int(row.find('a', {'data-bind': 'text: total_labelled, click: $parent.searchByLabels'}).text)
        if used_for == 'All' and usage_count != 0:
            return self._get_all_usage_counts(row_idx)
        else:
            return {
                used_for: usage_count
            }

    def _get_all_usage_counts(self, row_idx: int) -> dict:
        with LabelModal(self._browser, row_idx) as label_modal:
            return label_modal.get_usage_counts()

    def _get_first_name(self, row: BeautifulSoup):
        first_name = row.find('a', {'data-bind': 'text: first_name, click: $parent.searchByLabels'}).text
        if not first_name:
            return None
        else:
            return first_name

    def _get_last_name(self, row: BeautifulSoup):
        last_name = row.find('a', {'data-bind': 'text: last_name, click: $parent.searchByLabels'}).text
        if not last_name:
            return None
        else:
            return last_name

    def _validate_url(self, url):
        """
        Ensure that the given URL is a valid URL.

        Since the URL is not entered by the user, it is always valid.

        :param url: the url to validate
        :type url: str
        """
        return

    def _wait_until_page_is_loaded(self):
        """Wait until the page has finished loading."""
        self._browser.wait_until_element_exists_by_xpath('//tbody[@data-bind="foreach: institution_labels"]/tr')



import unittest

from autohandshake.src.Pages import Page
from autohandshake import HandshakeBrowser, UserType
from autohandshake.src.constants import BASE_URL
from autohandshake.tests import TestSession
from autohandshake.src.exceptions import InvalidUserTypeError


class TestPage(unittest.TestCase):

    def test_throws_error_when_required_methods_not_implemented(self):
        with self.assertRaises(TypeError):
            class SomePage(Page):
                pass

            some_page = SomePage(BASE_URL, HandshakeBrowser())

    def test_throws_error_when_method_called_with_incorrect_user_type(self):

        class TestPage(Page):

            def __init__(self, browser):
                super().__init__(BASE_URL, browser)

            @Page.require_user_type(UserType.STAFF)
            def staff_method(self):
                pass

            @Page.require_user_type(UserType.STUDENT)
            def student_method(self):
                pass

            @Page.require_user_type(UserType.EMPLOYER)
            def employer_method(self):
                pass

            def _validate_url(self, url):
                pass

            def _wait_until_page_is_loaded(self):
                pass

        with TestSession() as browser:
            self.assertEqual(UserType.STAFF, browser.user_type)
            test_page = TestPage(browser)
            with self.assertRaises(InvalidUserTypeError):
                test_page.student_method()
            with self.assertRaises(InvalidUserTypeError):
                test_page.employer_method()

            browser.switch_users(UserType.STUDENT)
            self.assertEqual(UserType.STUDENT, browser.user_type)

            with self.assertRaises(InvalidUserTypeError):
                test_page.employer_method()
            with self.assertRaises(InvalidUserTypeError):
                test_page.staff_method()
            try:
                test_page.student_method()
            except InvalidUserTypeError:
                browser.switch_users(UserType.STAFF)
                self.fail("Using an approved method still threw an error.")
            browser.switch_users(UserType.STAFF)



"""
Since the contents of the Handshake waiting room is a *highly* time-and-day
dependant thing, and is no way to create records to test against without
polluting the live waiting room, I do not believe it is possible to write
automated tests for it. Therefore, this file is to serve as a manual testing
"sandbox" of sorts o allow for testing against whatever the waiting room happens
to contain that day.
"""
from autohandshake import WaitingRoomPage
from autohandshake.tests import TestSession
import time

if __name__ == '__main__':
    with TestSession() as browser:
        waiting_room = WaitingRoomPage(browser)
        print(waiting_room.get_checkin_data())
        time.sleep(300)



import json
import os
import unittest
from datetime import date, datetime

from autohandshake.src.Pages import InsightsPage, FileType
from autohandshake.src.exceptions import InvalidURLError, NoSuchElementError
from autohandshake.tests import TestSession, download_dir


class TestInsightsPage(unittest.TestCase):

    def test_error_is_thrown_given_malformed_url(self):
        malformed_url = 'https://app.joinhandshake.com/anlytics/explore_embed?insights_page=ZXhwbG9yZS9nZW5lcmF0ZWRfaGFuZHNoYWtlX3Byb2R1Y3Rpb24vc3R1ZGVudHM_cWlkPXRoRnM0QnUyV2JKTHhQMDZ1dUpBT0QmZW1iZWRfZG9tYWluPWh0dHBzOiUyRiUyRmFwcC5qb2luaGFuZHNoYWtlLmNvbSZ0b2dnbGU9Zmls'
        malformed_query_param = 'fj4WQD2fhj$dsk2'
        with self.assertRaises(InvalidURLError):
            with TestSession() as browser:
                InsightsPage(malformed_url, browser)
        with self.assertRaises(InvalidURLError):
            with TestSession() as browser:
                InsightsPage(malformed_query_param, browser)

    def test_error_is_thrown_given_invalid_query_param(self):
        invalid_query = 'https://app.joinhandshake.com/analytics/explore_embed?insights_page=ZXhwbG9yZS9nZW5lcmF0ZWRfaGFuZHdb2R1Y3Rpb24vc3R1ZGVudHM_cWlkPXRoRnM0QnUyV2JKTHhQMDZ1dUpBT0QmZW1iZWRfZG9tYWluPWh0dHBzOiUyRiUyRmFwcC5qb2luaGFuZHNoYWtlLmNvbSZ0b2dnbGU9ZmlsLHBpaw=='
        with self.assertRaises(InvalidURLError):
            with TestSession() as browser:
                InsightsPage(invalid_query, browser)

    def test_no_error_is_thrown_given_saved_report_link_format(self):
        valid_query = 'https://app.joinhandshake.com/analytics/reports/3290'
        with TestSession() as browser:
            InsightsPage(valid_query, browser)

    def test_get_data(self):
        valid_query = 'https://app.joinhandshake.com/analytics/explore_embed?insights_page=ZXhwbG9yZS9nZW5lcmF0ZWRfaGFuZHNoYWtlX3Byb2R1Y3Rpb24vY2FyZWVyX3NlcnZpY2Vfc3RhZmZzP3FpZD1oUGFSSldmQWpyQjFqcDYyd3FCaWh2JmVtYmVkX2RvbWFpbj1odHRwczolMkYlMkZhcHAuam9pbmhhbmRzaGFrZS5jb20mdG9nZ2xlPWZpbA=='
        with TestSession() as browser:
            insights = InsightsPage(valid_query, browser)
            expected = [{'Career Service Staffs First Name': 'Christopher',
                         'Career Service Staffs Last Name': 'Edwards'},
                        {'Career Service Staffs First Name': 'David',
                         'Career Service Staffs Last Name': ' Le'},
                        {'Career Service Staffs First Name': 'Ella',
                         'Career Service Staffs Last Name': 'Stern'},
                        {'Career Service Staffs First Name': 'Farouk',
                         'Career Service Staffs Last Name': 'Dey'},
                        {'Career Service Staffs First Name': 'Joy',
                         'Career Service Staffs Last Name': 'Saunders'},
                        {'Career Service Staffs First Name': 'Ted',
                         'Career Service Staffs Last Name': 'Shaprow'},
                        {'Career Service Staffs First Name': 'Troy',
                         'Career Service Staffs Last Name': 'Parker: Technology & Innovation'}]
            actual = insights.get_data()
            actual.sort(key=lambda x: x['Career Service Staffs First Name'])
            self.assertEqual(expected, actual)

    def test_get_data_with_custom_limit(self):
        query = 'https://app.joinhandshake.com/analytics/explore_embed?insights_page=ZXhwbG9yZS9nZW5lcmF0ZWRfaGFuZHNoYWtlX3Byb2R1Y3Rpb24vY2FyZWVyX3NlcnZpY2Vfc3RhZmZzP3FpZD1oUGFSSldmQWpyQjFqcDYyd3FCaWh2JmVtYmVkX2RvbWFpbj1odHRwczolMkYlMkZhcHAuam9pbmhhbmRzaGFrZS5jb20mdG9nZ2xlPWZpbA=='
        with TestSession() as browser:
            insights = InsightsPage(query, browser)
            expected = [{'Career Service Staffs First Name': 'David',
                         'Career Service Staffs Last Name': ' Le'},
                        {'Career Service Staffs First Name': 'Ella',
                         'Career Service Staffs Last Name': 'Stern'},
                        {'Career Service Staffs First Name': 'Joy',
                         'Career Service Staffs Last Name': 'Saunders'}]
            actual = insights.get_data(limit=3)
            actual.sort(key=lambda x: x['Career Service Staffs First Name'])
            self.assertEqual(expected, actual)

    def test_download_file_works_with_valid_download_dir(self):
        valid_query = 'https://app.joinhandshake.com/analytics/explore_embed?insights_page=ZXhwbG9yZS9nZW5lcmF0ZWRfaGFuZHNoYWtlX3Byb2R1Y3Rpb24vc3R1ZGVudHM_cWlkPWtmaHNhdzRvODh0QnFPY1FxQ1NCNzYmZW1iZWRfZG9tYWluPWh0dHBzOiUyRiUyRmFwcC5qb2luaGFuZHNoYWtlLmNvbSZ0b2dnbGU9ZmlsLHBpaw=='
        file_name = 'test_file_834898330.xlsx'
        expected_filepath = os.path.join(download_dir, file_name)
        with TestSession() as browser:
            insights = InsightsPage(valid_query, browser)
            downloaded_filepath = insights.download_file(download_dir, file_name, FileType.EXCEL)
            self.assertEqual(expected_filepath, downloaded_filepath)
            self.assertTrue(os.path.exists(expected_filepath))
            os.remove(expected_filepath)

    def test_download_file_with_custom_limit(self):
        query = 'https://app.joinhandshake.com/analytics/explore_embed?insights_page=ZXhwbG9yZS9nZW5lcmF0ZWRfaGFuZHNoYWtlX3Byb2R1Y3Rpb24vY2FyZWVyX3NlcnZpY2Vfc3RhZmZzP3FpZD1oUGFSSldmQWpyQjFqcDYyd3FCaWh2JmVtYmVkX2RvbWFpbj1odHRwczolMkYlMkZhcHAuam9pbmhhbmRzaGFrZS5jb20mdG9nZ2xlPWZpbA=='
        file_name = 'test_file_2589230.json_label'
        expected_filepath = os.path.join(download_dir, file_name)
        with TestSession() as browser:
            insights = InsightsPage(query, browser)
            downloaded_filepath = insights.download_file(download_dir, file_name, FileType.JSON, limit=3)
        expected = [{'Career Service Staffs First Name': 'Christopher',
                     'Career Service Staffs Last Name': 'Edwards'},
                    {'Career Service Staffs First Name': 'David',
                     'Career Service Staffs Last Name': ' Le'},
                    {'Career Service Staffs First Name': 'Ella',
                     'Career Service Staffs Last Name': 'Stern'}]
        self.assertTrue(os.path.exists(expected_filepath))
        with open(downloaded_filepath) as file:
            actual = json.load(file)
            actual.sort(key=lambda x: x['Career Service Staffs First Name'])
        self.assertEqual(expected, actual)
        os.remove(downloaded_filepath)

    def test_set_date_range_filter(self):
        def test_date_field(insights: InsightsPage, field: dict, new_start: date, new_end: date):
            """Helper function to test a given date field"""
            self.assertEqual(field['old_start_date'],
                             insights._browser.get_element_attribute_by_xpath(field['start_xpath'], 'value'))
            self.assertEqual(field['old_end_date'],
                             insights._browser.get_element_attribute_by_xpath(field['end_xpath'], 'value'))
            insights.set_date_range_filter(field['category'], field['title'], new_start, new_end)
            self.assertEqual(new_start.strftime('%Y-%m-%d'),
                             insights._browser.get_element_attribute_by_xpath(field['start_xpath'], 'value'))
            self.assertEqual(new_end.strftime('%Y-%m-%d'),
                             insights._browser.get_element_attribute_by_xpath(field['end_xpath'], 'value'))

        jobs_insights = 'https://app.joinhandshake.com/analytics/explore_embed?insights_page=ZXhwbG9yZS9nZW5lcmF0ZWRfaGFuZHNoYWtlX3Byb2R1Y3Rpb24vam9icz9xaWQ9UVJ3OVZRR2xORHVnYnVQZWJtYVI3diZlbWJlZF9kb21haW49aHR0cHM6JTJGJTJGYXBwLmpvaW5oYW5kc2hha2UuY29tJnRvZ2dsZT1maWw='
        field1 = {'category': 'Postings', 'title': 'Created At Date',
                  'old_start_date': '2007-05-09', 'old_end_date': '2014-05-07',
                  'start_xpath': '//*[@id="lk-embed-container"]/lk-explore-dataflux/div[2]/lk-explore-content/div/div/lk-filter-pane/lk-expandable-pane/div[2]/expandable-pane-content/lk-query-filters/table/tbody/tr[2]/td[3]/lk-filter/table/tbody/tr/td[2]/span[2]/span[1]/input',
                  'end_xpath': '//*[@id="lk-embed-container"]/lk-explore-dataflux/div[2]/lk-explore-content/div/div/lk-filter-pane/lk-expandable-pane/div[2]/expandable-pane-content/lk-query-filters/table/tbody/tr[2]/td[3]/lk-filter/table/tbody/tr/td[2]/span[3]/span/input'}

        field2 = {'category': 'Postings', 'title': 'Expiration Date Date',
                  'old_start_date': '2016-03-28', 'old_end_date': '2017-08-28',
                  'start_xpath': '//*[@id="lk-embed-container"]/lk-explore-dataflux/div[2]/lk-explore-content/div/div/lk-filter-pane/lk-expandable-pane/div[2]/expandable-pane-content/lk-query-filters/table/tbody/tr[3]/td[3]/lk-filter/table/tbody/tr/td[2]/span[2]/span[1]/input',
                  'end_xpath': '//*[@id="lk-embed-container"]/lk-explore-dataflux/div[2]/lk-explore-content/div/div/lk-filter-pane/lk-expandable-pane/div[2]/expandable-pane-content/lk-query-filters/table/tbody/tr[3]/td[3]/lk-filter/table/tbody/tr/td[2]/span[3]/span/input'}

        field3 = {'category': 'Jobs', 'title': 'Start Date Date',
                  'old_start_date': '2019-04-02', 'old_end_date': '2019-04-16',
                  'start_xpath': '//*[@id="lk-embed-container"]/lk-explore-dataflux/div[2]/lk-explore-content/div/div/lk-filter-pane/lk-expandable-pane/div[2]/expandable-pane-content/lk-query-filters/table/tbody/tr[1]/td[3]/lk-filter/table/tbody/tr/td[2]/span[2]/span[1]/input',
                  'end_xpath': '//*[@id="lk-embed-container"]/lk-explore-dataflux/div[2]/lk-explore-content/div/div/lk-filter-pane/lk-expandable-pane/div[2]/expandable-pane-content/lk-query-filters/table/tbody/tr[1]/td[3]/lk-filter/table/tbody/tr/td[2]/span[3]/span/input'}

        with TestSession() as browser:
            insights = InsightsPage(jobs_insights, browser)
            test_date_field(insights, field1, datetime(2007, 5, 9), datetime(2014, 5, 7))
            test_date_field(insights, field1, datetime.strptime('10/31/2017', '%m/%d/%Y').date(),
                            datetime.strptime('12/03/2017', '%m/%d/%Y').date())
            test_date_field(insights, field2, datetime.strptime('12/02/2008', '%m/%d/%Y').date(),
                            datetime.strptime('05/29/2019', '%m/%d/%Y').date())
            test_date_field(insights, field3, datetime.strptime('01/02/2013', '%m/%d/%Y').date(),
                            datetime.strptime('02/03/2014', '%m/%d/%Y').date())


class TestInsightsPageDownloadModal(unittest.TestCase):

    def test_error_is_thrown_when_selecting_download_file_type_outside_of_dialogue_box(self):
        valid_query = 'https://app.joinhandshake.com/analytics/explore_embed?insights_page=ZXhwbG9yZS9nZW5lcmF0ZWRfaGFuZHNoYWtlX3Byb2R1Y3Rpb24vc3R1ZGVudHM_cWlkPXRoRnM0QnUyV2JKTHhQMDZ1dUpBT0QmZW1iZWRfZG9tYWluPWh0dHBzOiUyRiUyRmFwcC5qb2luaGFuZHNoYWtlLmNvbSZ0b2dnbGU9ZmlsLHBpaw=='
        with TestSession() as browser:
            insights = InsightsPage(valid_query, browser)
            with self.assertRaises(NoSuchElementError):
                insights.modal.set_download_file_type('excel')

    def test_no_error_is_thrown_selecting_valid_download_file_type(self):
        valid_query = 'https://app.joinhandshake.com/analytics/explore_embed?insights_page=ZXhwbG9yZS9nZW5lcmF0ZWRfaGFuZHNoYWtlX3Byb2R1Y3Rpb24vc3R1ZGVudHM_cWlkPXRoRnM0QnUyV2JKTHhQMDZ1dUpBT0QmZW1iZWRfZG9tYWluPWh0dHBzOiUyRiUyRmFwcC5qb2luaGFuZHNoYWtlLmNvbSZ0b2dnbGU9ZmlsLHBpaw=='
        with TestSession() as browser:
            insights = InsightsPage(valid_query, browser)
            insights.modal.open()
            insights.modal.set_download_file_type(FileType.TXT)
            self.assertEqual(FileType.TXT, insights.modal.get_download_file_type())
            insights.modal.set_download_file_type(FileType.CSV)
            self.assertEqual(FileType.CSV, insights.modal.get_download_file_type())
            insights.modal.set_download_file_type(FileType.JSON)
            self.assertEqual(FileType.JSON_LABEL, insights.modal.get_download_file_type())
            insights.modal.set_download_file_type(FileType.HTML)
            self.assertEqual(FileType.HTML, insights.modal.get_download_file_type())
            insights.modal.set_download_file_type(FileType.MARKDOWN)
            self.assertEqual(FileType.MARKDOWN, insights.modal.get_download_file_type())
            insights.modal.set_download_file_type(FileType.PNG)
            self.assertEqual(FileType.PNG, insights.modal.get_download_file_type())

    def test_set_limit_to_all_results(self):
        valid_query = 'https://app.joinhandshake.com/analytics/explore_embed?insights_page=ZXhwbG9yZS9nZW5lcmF0ZWRfaGFuZHNoYWtlX3Byb2R1Y3Rpb24vc3R1ZGVudHM_cWlkPXRoRnM0QnUyV2JKTHhQMDZ1dUpBT0QmZW1iZWRfZG9tYWluPWh0dHBzOiUyRiUyRmFwcC5qb2luaGFuZHNoYWtlLmNvbSZ0b2dnbGU9ZmlsLHBpaw=='
        with TestSession() as browser:
            insights = InsightsPage(valid_query, browser)
            insights.modal.open()
            insights.modal.set_limit_to_all_results()
            self.assertTrue(browser.element_is_selected_by_xpath(
                "//input[@name='qr-export-modal-limit' and @value='all']"))
            insights.modal.set_limit_to_all_results(True)
            self.assertTrue(browser.element_is_selected_by_xpath(
                "//input[@ng-model='queryDownloadController.runUnsorted']"))

    def test_set_custom_limit(self):
        valid_query = 'https://app.joinhandshake.com/analytics/explore_embed?insights_page=ZXhwbG9yZS9nZW5lcmF0ZWRfaGFuZHNoYWtlX3Byb2R1Y3Rpb24vc3R1ZGVudHM_cWlkPXRoRnM0QnUyV2JKTHhQMDZ1dUpBT0QmZW1iZWRfZG9tYWluPWh0dHBzOiUyRiUyRmFwcC5qb2luaGFuZHNoYWtlLmNvbSZ0b2dnbGU9ZmlsLHBpaw=='
        with TestSession(60) as browser:
            insights = InsightsPage(valid_query, browser)
            insights.modal.open()
            insights.modal.set_custom_limit(357)
            self.assertTrue(browser.element_is_selected_by_xpath(
                "//input[@name='qr-export-modal-limit' and @value='custom']"))
            self.assertEqual(browser.get_element_attribute_by_xpath(
                "//input[@name='customExportLimit']", "value"), '357')

    def test_set_file_name(self):
        valid_query = 'https://app.joinhandshake.com/analytics/explore_embed?insights_page=ZXhwbG9yZS9nZW5lcmF0ZWRfaGFuZHNoYWtlX3Byb2R1Y3Rpb24vc3R1ZGVudHM_cWlkPXRoRnM0QnUyV2JKTHhQMDZ1dUpBT0QmZW1iZWRfZG9tYWluPWh0dHBzOiUyRiUyRmFwcC5qb2luaGFuZHNoYWtlLmNvbSZ0b2dnbGU9ZmlsLHBpaw=='
        with TestSession() as browser:
            insights = InsightsPage(valid_query, browser)
            insights.modal.open()
            insights.modal.set_file_name('data_file.csv')
            self.assertEqual(browser.get_element_attribute_by_xpath(
                "//input[@name='customExportFilename']", "value"), 'data_file.csv')

            insights.modal.set_file_name('wrong_extension.json')
            self.assertEqual(browser.get_element_attribute_by_xpath(
                "//input[@name='customExportFilename']", "value"), 'wrong_extension.json.csv')

    def test_set_file_name_throws_error_for_invalid_file_name(self):
        valid_query = 'https://app.joinhandshake.com/analytics/explore_embed?insights_page=ZXhwbG9yZS9nZW5lcmF0ZWRfaGFuZHNoYWtlX3Byb2R1Y3Rpb24vc3R1ZGVudHM_cWlkPXRoRnM0QnUyV2JKTHhQMDZ1dUpBT0QmZW1iZWRfZG9tYWluPWh0dHBzOiUyRiUyRmFwcC5qb2luaGFuZHNoYWtlLmNvbSZ0b2dnbGU9ZmlsLHBpaw=='
        with TestSession() as browser:
            insights = InsightsPage(valid_query, browser)
            insights.modal.open()
            with self.assertRaises(ValueError):
                insights.modal.set_file_name('')

    def test_download_file_works_with_valid_download_dir(self):
        valid_query = 'https://app.joinhandshake.com/analytics/explore_embed?insights_page=ZXhwbG9yZS9nZW5lcmF0ZWRfaGFuZHNoYWtlX3Byb2R1Y3Rpb24vc3R1ZGVudHM_cWlkPWtmaHNhdzRvODh0QnFPY1FxQ1NCNzYmZW1iZWRfZG9tYWluPWh0dHBzOiUyRiUyRmFwcC5qb2luaGFuZHNoYWtlLmNvbSZ0b2dnbGU9ZmlsLHBpaw=='
        file_name = 'test_file_123456789.csv'
        file_path = download_dir + file_name
        with TestSession() as browser:
            insights = InsightsPage(valid_query, browser)
            insights.modal.open()
            insights.modal.set_file_name(file_name)
            self.assertEqual(file_path, insights.modal.click_download_button(download_dir))
            self.assertTrue(os.path.exists(file_path))
            os.remove(file_path)

    def test_download_file_throws_error_with_invalid_download_dir(self):
        valid_query = 'https://app.joinhandshake.com/analytics/explore_embed?insights_page=ZXhwbG9yZS9nZW5lcmF0ZWRfaGFuZHNoYWtlX3Byb2R1Y3Rpb24vc3R1ZGVudHM_cWlkPWtmaHNhdzRvODh0QnFPY1FxQ1NCNzYmZW1iZWRfZG9tYWluPWh0dHBzOiUyRiUyRmFwcC5qb2luaGFuZHNoYWtlLmNvbSZ0b2dnbGU9ZmlsLHBpaw=='
        file_name = 'test_file_123456789.csv'
        with TestSession() as browser:
            insights = InsightsPage(valid_query, browser)
            insights.modal.open()
            insights.modal.set_file_name(file_name)
            with self.assertRaises(RuntimeError):
                insights.modal.click_download_button('a/fake/download/dir/', 10)

    def test_open_in_browser_with_valid_file_type(self):
        expected = [
            {
                'Career Service Staffs First Name': "David",
                'Career Service Staffs Last Name': " Le"
            },
            {
                'Career Service Staffs First Name': "Ella",
                'Career Service Staffs Last Name': "Stern"
            },
            {
                'Career Service Staffs First Name': "Joy",
                'Career Service Staffs Last Name': "Saunders"
            },
            {
                'Career Service Staffs First Name': "Ted",
                'Career Service Staffs Last Name': "Shaprow"
            },
            {
                'Career Service Staffs First Name': "Troy",
                'Career Service Staffs Last Name': "Parker"
            }
        ]
        valid_query = 'https://app.joinhandshake.com/analytics/explore_embed?insights_page=ZXhwbG9yZS9nZW5lcmF0ZWRfaGFuZHNoYWtlX3Byb2R1Y3Rpb24vY2FyZWVyX3NlcnZpY2Vfc3RhZmZzP3FpZD1oUGFSSldmQWpyQjFqcDYyd3FCaWh2JmVtYmVkX2RvbWFpbj1odHRwczolMkYlMkZhcHAuam9pbmhhbmRzaGFrZS5jb20mdG9nZ2xlPWZpbA=='
        with TestSession() as browser:
            insights = InsightsPage(valid_query, browser)
            insights.modal.open()
            insights.modal.set_download_file_type(FileType.JSON)
            json_data = insights.modal.click_open_in_browser()
            self.assertEqual(expected.sort(key=lambda x: x['Career Service Staffs First Name']),
                             json_data.sort(key=lambda x: x['Career Service Staffs First Name']))

    def test_open_in_browser_throws_error_if_not_json(self):
        valid_query = 'https://app.joinhandshake.com/analytics/explore_embed?insights_page=ZXhwbG9yZS9nZW5lcmF0ZWRfaGFuZHNoYWtlX3Byb2R1Y3Rpb24vY2FyZWVyX3NlcnZpY2Vfc3RhZmZzP3FpZD1oUGFSSldmQWpyQjFqcDYyd3FCaWh2JmVtYmVkX2RvbWFpbj1odHRwczolMkYlMkZhcHAuam9pbmhhbmRzaGFrZS5jb20mdG9nZ2xlPWZpbA=='
        with TestSession() as browser:
            insights = InsightsPage(valid_query, browser)
            insights.modal.open()
            insights.modal.set_download_file_type(FileType.EXCEL)
            with self.assertRaises(RuntimeError):
                insights.modal.click_open_in_browser()



import unittest

from autohandshake import SurveyPage
from autohandshake.tests import TestSession
import csv

TEST_SURVEY = 33726


def csv_to_list_of_dicts(filepath):
    with open(filepath) as file:
        return [{k: v for k, v in row.items()} for row in csv.DictReader(file, skipinitialspace=True)]


class TestSurveyPage(unittest.TestCase):

    def test_get_survey_data(self):
        expected = [
            {
                'Name': 'Christopher Edwards',
                'Date': '2019-08-07 17:58:01 UTC',
                'Email Address': 'cedwar42@jhu.edu',
                'What is your name?': 'John',
                'What is your quest?': 'Option B',
                'Describe your dog:': 'She is small and cute.',
                'Which of the following?': 'a, c',
                'How likely are you?': '3'
            },
            {
                'Name': 'Jasmine Miller',
                'Date': '2019-08-07 17:59:25 UTC',
                'Email Address': 'rshille1@jhu.edu',
                'What is your name?': 'Becca',
                'What is your quest?': 'Option Three',
                'Describe your dog:': 'I have a cat',
                'Which of the following?': 'e',
                'How likely are you?': '9'
            },
            {
                'Name': 'Aleanairis Nunez',
                'Date': '2019-08-07 18:00:50 UTC',
                'Email Address': 'anunez5@jhu.edu',
                'What is your name?': 'Aly',
                'What is your quest?': 'Option 1',
                'Describe your dog:': 'I might have a dog?',
                'Which of the following?': 'a, b, c, d, e',
                'How likely are you?': '5'
            },
        ]

        with TestSession() as browser:
            page = SurveyPage(TEST_SURVEY, browser)
            filepath = page.download_responses(TestSession.download_dir)
            actual = csv_to_list_of_dicts(filepath)
            self.assertEqual(expected, actual)



from typing import List

from bs4 import BeautifulSoup

from autohandshake.src.Pages.Page import Page
from autohandshake.src.HandshakeBrowser import HandshakeBrowser, UserType
from autohandshake.src.constants import BASE_URL


class EventPage(Page):
    """
    A specific Event page in Handshake.
    """

    def __init__(self, event_id: int, browser: HandshakeBrowser):
        """
        :param event_id: the id of the event page to visit
        :type event_id: int
        :param browser: a logged-in HandshakeBrowser
        :type browser: HandshakeBrowser
        """
        super().__init__(f'{BASE_URL}/events/{event_id}', browser)

    @Page.require_user_type(UserType.STAFF)
    def get_invited_schools(self) -> List[str]:
        """
        Get a list of the names of schools invited to the event
        """
        school_elements = BeautifulSoup(self._browser.get_element_attribute_by_xpath(
            '//div[h4[text()="Invited Schools"]]', 'innerHTML'),
            'html.parser').find_all('img')
        return [school.get('title') for school in school_elements]

    def _validate_url(self, url):
        """
        Ensure that the given URL is a valid URL.

        Since the URL is not entered by the user, it is always valid.

        :param url: the url to validate
        :type url: str
        """
        return

    def _wait_until_page_is_loaded(self):
        """Wait until the page has finished loading."""
        self._browser.wait_until_element_exists_by_xpath('//div[@class="row attendees-lists"]')
